@@ -19,17 +19,19 @@ from s3ql.common import ChecksumError
 import ConfigParser
 import bz2
 import cPickle as pickle
+import errno
 import hashlib
 import hmac
 import httplib
 import logging
 import lzma
+import math
 import os
 import re
+import socket
 import stat
 import struct
 import sys
-import math
 import threading
 import time
 import zlib
@@ -89,6 +91,31 @@ True.
 '''
     return wrapped
 
+def is_temp_network_error(exc):
+    '''Return true if *exc* represents a potentially temporary network problem'''
+
+    if isinstance(exc, (httplib.IncompleteRead, socket.timeout)):
+        return True
+     
+    # Server closed connection
+    elif (isinstance(exc, httplib.BadStatusLine)
+          and (not exc.line or exc.line == "''")):
+        return True
+
+    elif (isinstance(exc, IOError) and
+          exc.errno in (errno.EPIPE, errno.ECONNRESET, errno.ETIMEDOUT,
+                        errno.EINTR)):
+        return True
+
+    # Formally this is a permanent error. However, it may also indicate
+    # that there is currently no network connection to the DNS server
+    elif (isinstance(exc, socket.gaierror) 
+          and exc.errno in (socket.EAI_AGAIN, socket.EAI_NONAME)):
+        return True 
+              
+    return False
+    
+    
 def http_connection(hostname, port, ssl=False):
     '''Return http connection to *hostname*:*port*
     
@@ -851,7 +878,10 @@ class DecryptFilter(AbstractInputFilter):
                 self.hmac.update(inbuf)
                 outbuf += inbuf
                 break
-
+            elif len(inbuf) <= self.remaining + self.off_size:
+                inbuf += self.fh.read(self.off_size)
+                continue
+                
             outbuf += inbuf[:self.remaining]
             self.hmac.update(inbuf[:self.remaining + self.off_size])
             paket_size = struct.unpack(b'<I', inbuf[self.remaining
@@ -1025,11 +1055,11 @@ class NoSuchObject(Exception):
     def __str__(self):
         return 'Backend does not have anything stored under key %r' % self.key
 
-class DanglingStorageURL(Exception):
+class DanglingStorageURLError(Exception):
     '''Raised if the backend can't store data at the given location'''
 
     def __init__(self, loc):
-        super(DanglingStorageURL, self).__init__()
+        super(DanglingStorageURLError, self).__init__()
         self.loc = loc
 
     def __str__(self):
@@ -1173,7 +1203,7 @@ def get_backend_factory(options, plain=False):
         # (e.g. wrong credentials)
         _ = backend['s3ql_passphrase']
         
-    except DanglingStorageURL as exc:
+    except DanglingStorageURLError as exc:
         raise QuietError(str(exc))   
     
     except AuthorizationError:
