@@ -1201,25 +1201,28 @@ def main(args=None):
         log.warn('File system was marked as clean, yet fsck found problems.')
         log.warn('Please report this to the S3QL mailing list, http://groups.google.com/group/s3ql')
 
-    cycle_metadata(backend)
     param['needs_fsck'] = False
     param['last_fsck'] = time.time()
     param['last-modified'] = time.time()
 
     log.info('Dumping metadata...')
-    fh = tempfile.TemporaryFile()
-    dump_metadata(db, fh)
-    def do_write(obj_fh):
-        fh.seek(0)
-        stream_write_bz2(fh, obj_fh)
-        return obj_fh
-
-    log.info("Compressing and uploading metadata...")
-    obj_fh = backend.perform_write(do_write, "s3ql_metadata", metadata=param,
-                                  is_compressed=True)
+    with tempfile.TemporaryFile() as fh:
+        dump_metadata(db, fh)
+        def do_write(obj_fh):
+            fh.seek(0)
+            stream_write_bz2(fh, obj_fh)
+            return obj_fh
+    
+        log.info("Compressing and uploading metadata...")
+        obj_fh = backend.perform_write(do_write, "s3ql_metadata_new", metadata=param,
+                                      is_compressed=True)
     log.info('Wrote %.2f MiB of compressed metadata.', obj_fh.get_obj_size() / 1024 ** 2)
-    pickle.dump(param, open(cachepath + '.params', 'wb'), 2)
+    log.info('Cycling metadata backups...')
+    cycle_metadata(backend)
+    with open(cachepath + '.params', 'wb') as fh:
+        pickle.dump(param, fh, 2)
 
+    log.info('Cleaning up local metadata...')
     db.execute('ANALYZE')
     db.execute('VACUUM')
     db.close()
