@@ -114,18 +114,23 @@ def main(args=None):
                                upload_event=metadata_upload_thread.event)
     
     log.info('Mounting filesystem...')
-    llfuse.init(operations, options.mountpoint, get_fuse_opts(options))
-
-    if not options.fg:
-        if stdout_log_handler:
-            logging.getLogger().removeHandler(stdout_log_handler)
-        daemonize(options.cachedir)
-
-    exc_info = setup_exchook()
-
-    # After we start threads, we must be sure to terminate them
-    # or the process will hang 
     try:
+        llfuse.init(operations, options.mountpoint, get_fuse_opts(options))
+    except RuntimeError as exc:
+        raise QuietError(str(exc))
+
+    # From here on, we have to clean-up the mountpoint and
+    # terminate started threads.
+    try:
+        if not options.fg:
+            if stdout_log_handler:
+                logging.getLogger().removeHandler(stdout_log_handler)
+            daemonize(options.cachedir)
+
+        exc_info = setup_exchook()
+
+        mark_metadata_dirty(backend, cachepath, param)
+        
         block_cache.init(options.threads)
         metadata_upload_thread.start()
         commit_thread.start()
@@ -216,7 +221,6 @@ def main(args=None):
             param['seq_no'] -= 1
             pickle.dump(param, open(cachepath + '.params', 'wb'), 2)
         elif seq_no == param['seq_no']:
-            cycle_metadata(backend)
             param['last-modified'] = time.time()
 
             log.info('Dumping metadata...')
@@ -228,10 +232,13 @@ def main(args=None):
                 return obj_fh
 
             log.info("Compressing and uploading metadata...")
-            obj_fh = backend.perform_write(do_write, "s3ql_metadata", metadata=param,
-                                          is_compressed=True)
+            obj_fh = backend.perform_write(do_write, "s3ql_metadata_new",
+                                           metadata=param, is_compressed=True)
             log.info('Wrote %.2f MiB of compressed metadata.', obj_fh.get_obj_size() / 1024 ** 2)
-            pickle.dump(param, open(cachepath + '.params', 'wb'), 2)
+            log.info('Cycling metadata backups...')
+            cycle_metadata(backend)
+            with open(cachepath + '.params', 'wb') as fh:
+                pickle.dump(param, fh, 2)
         else:
             log.error('Remote metadata is newer than local (%d vs %d), '
                       'refusing to overwrite!', seq_no, param['seq_no'])
@@ -243,6 +250,7 @@ def main(args=None):
                         os.rename(name + '.%d' % i, name + '.%d' % (i + 1))
                 os.rename(name, name + '.0')
 
+    log.info('Cleaning up local metadata...')
     db.execute('ANALYZE')
     db.execute('VACUUM')
     db.close()
@@ -258,6 +266,8 @@ def main(args=None):
             p.sort_stats('time')
             p.print_stats(50)
 
+    log.info('All done.')
+    
 def determine_threads(options):
     '''Return optimum number of upload threads'''
 
@@ -345,14 +355,20 @@ def get_metadata(backend, cachepath):
         tmpfh.seek(0)
         db = restore_metadata(tmpfh, cachepath + '.db')
 
-    # Increase metadata sequence no 
+    with open(cachepath + '.params', 'wb') as fh:
+        pickle.dump(param, fh, 2)
+            
+    return (param, db)
+
+def mark_metadata_dirty(backend, cachepath, param):
+    '''Mark metadata as dirty and increase sequence number'''
+    
     param['seq_no'] += 1
     param['needs_fsck'] = True
     backend['s3ql_seq_no_%d' % param['seq_no']] = 'Empty'
     pickle.dump(param, open(cachepath + '.params', 'wb'), 2)
     param['needs_fsck'] = False
 
-    return (param, db)
 
 def get_fuse_opts(options):
     '''Return fuse options for given command line options'''
@@ -537,7 +553,6 @@ class MetadataUploadThread(Thread):
                     fh.close()
                     continue
 
-                cycle_metadata(backend)
                 fh.seek(0)
                 self.param['last-modified'] = time.time()
 
@@ -548,9 +563,12 @@ class MetadataUploadThread(Thread):
                     stream_write_bz2(fh, obj_fh)
                     return obj_fh
                 log.info("Compressing and uploading metadata...")
-                obj_fh = backend.perform_write(do_write, "s3ql_metadata", metadata=self.param,
-                                              is_compressed=True)
-                log.info('Wrote %.2f MiB of compressed metadata.', obj_fh.get_obj_size() / 1024 ** 2)
+                obj_fh = backend.perform_write(do_write, "s3ql_metadata_new",
+                                               metadata=self.param, is_compressed=True)
+                log.info('Wrote %.2f MiB of compressed metadata.',
+                         obj_fh.get_obj_size() / 1024 ** 2)
+                log.info('Cycling metadata backups...')
+                cycle_metadata(backend)
                 self.param['seq_no'] += 1
 
                 fh.close()
