@@ -12,7 +12,8 @@ import errno
 import os.path
 import subprocess
 import t4_fuse
-import tarfile
+from t4_fuse import populate_dir
+import sys
 import tempfile
 import unittest2 as unittest
 import shutil
@@ -37,20 +38,17 @@ class cpTests(t4_fuse.fuse_tests):
 
     def tst_cp(self):
 
-        # Extract tar
-        data_file = os.path.join(os.path.dirname(__file__), 'data.tar.bz2')
         tempdir = tempfile.mkdtemp()
         try:
-            tarfile.open(data_file).extractall(tempdir)
+            populate_dir(tempdir)
 
             # Rsync
             subprocess.check_call(['rsync', '-aHAX', tempdir + '/',
                                    os.path.join(self.mnt_dir, 'orig') + '/'])
 
             # copy
-            subprocess.check_call([os.path.join(t4_fuse.BASEDIR, 'bin', 's3qlcp'),
-                                   '--quiet',
-                                   os.path.join(self.mnt_dir, 'orig'),
+            subprocess.check_call([sys.executable, os.path.join(t4_fuse.BASEDIR, 'bin', 's3qlcp'),
+                                   '--quiet', os.path.join(self.mnt_dir, 'orig'),
                                    os.path.join(self.mnt_dir, 'copy')])
 
             # compare
